==> LL_utils-4.1.1.2-x86_64-Linux-RHEL6.tar.gz: var/www/ll/cgi-bin/on_update_panel.pl <==
#!/usr/bin/perl

# static char sccsid[] = "@(#)20 1.9 src/ll/config_editor/cgi-bin/on_update_panel.pl, ll.config, ll_rrur2, rrur21042a 1/12/10 03:38:07\n";

use strict;
use lib "/var/www/ll/cgi-bin";
use LLUtil;
use DBI;

my $query = $ENV{QUERY_STRING};
$query = unescape($query);

debug_log($query);

# type=normal&pk_name=nodeID&pk_id=27&name=abc&TLLR_CFGMachine-adapter_list=33&TLLR_CFGMachine-alias=33&TLLR_CFGMachine-cpu_speed_scale=33
# type=editable&pk_name=nodeID-class_name&pk_id=27=a&table=TLLR_CFGMachineClass&class_count=11&type=all_value&name=abc&class_name=a&class_count=11&class_name=b&class_count=22	
my @keywords = split('&',$query);
my $kw_count = scalar(@keywords);
my @type = split('=',$keywords[0]);

# create the db connection
my $dbh = get_dbh(0);
if(!defined($dbh))
{
	error_and_exit(DBI->errstr);
}

my @update_statements = ();
my $json_str = LLUtil::OUTPUT_HEADER;

if($type[1] eq "normal")
{
# type=normal&pk_name=nodeID&pk_id=27&name=abc&TLLR_CFGMachine-adapter_list=33&TLLR_CFGMachine-alias=33&TLLR_CFGMachine-cpu_speed_scale=33    
	my @tmp = split('=',$keywords[1]);	
	my $pk_name = $tmp[1];
	@tmp = split('=',$keywords[2]);
	my $pk_id = $tmp[1];
	@tmp = split('=',$keywords[3]);
	my $stanza_name = $tmp[1];
	my $table;
	my %key_map = ();
	for(my $i=4;$i<$kw_count;$i++)
	{
		@tmp = split('=',$keywords[$i]);
		my $value = $tmp[1];
		@tmp = split('-',$tmp[0]);
		$table = $tmp[0];
		my $key = $tmp[1];
		
		check_data($dbh,$table,$key,$value,$stanza_name,"normal_update");
		$key_map{$tmp[1]}=$value;
	}

	# find out if the record exists
	my $sql_str="select ".$pk_name." from ".$table." where ".$pk_name."=".$pk_id;
	my $sth=$dbh->prepare($sql_str);
	if(!defined($sth->execute()))
	{
		error_and_exit($dbh->errstr);
	}
	my($id);
	$sth->bind_columns(undef,\$id);
	my $existed = 0;
	if($sth->fetch())
	{
		$existed = 1;
	}
	
	# format the update SQL statement
	my $statement;
	if($existed == 1)
	{
		$statement = "update ".$table." set ";
		foreach my $key (keys %key_map)
		{
			$statement .= $key."='".$key_map{$key}."',";
		}
		$statement =~ s/,$//g;
		$statement .= " where ".$pk_name."=".$pk_id;
	}
	else
	{
		
		$statement = "insert into ".$table." set ".$pk_name."=".$pk_id.",";
		foreach my $key (keys %key_map)
		{
			$statement .= " ".$key."='".$key_map{$key}."',";
		}
		$statement =~ s/,$//g;
	}
	
	push(@update_statements, $statement);
	
	$json_str .= "OK";
	
}
elsif($type[1] eq "editable")
{
# type=editable&pk_name=nodeID-class_name&pk_id=27=a&table=TLLR_CFGMachineClass&class_count=11&type=all_value&name=abc&class_name=a&class_count=11&class_name=b&class_count=22	
	my @tmp = split('=',$keywords[1]);
	my @primary_key_names = split('-',$tmp[1]);	
	my @primary_key_values = split('=',$keywords[2]);
	shift(@primary_key_values);
	@tmp = split('=',$keywords[3]);
	my $table = $tmp[1];

	# format the update SQL statement
	my $statement = "update ".$table." set ";
	my $check = 0;
	for(my $i = 4; $i < $kw_count; $i++)
	{
		if($keywords[$i] eq "type=all_value")
		{
			$check = $i;
			last;
		}
		
		@tmp = split('=',$keywords[$i]);
		$statement .= $tmp[0]."='".$tmp[1]."',";		
	}

	# format the <keyword>,<value> pair for checking
	if($check > 0 && $kw_count > $check + 1)
	{
		@tmp = split('=',$keywords[$check + 1]);
		my $stanza_name = $tmp[1];

		@tmp = split('=',$keywords[$check + 2]);
		my $first_key_name = $tmp[0];
		my @columns = ();
		push(@columns, $primary_key_values[0]);
		push(@columns, $tmp[1]);
		
		my $keyword_string = "";
		for(my $i = $check+3; $i < $kw_count; $i++)
		{
			@tmp = split('=',$keywords[$i]);
			if($tmp[0] eq $first_key_name)
			{
				$keyword_string .= format_keywords($dbh,\@columns,$table)." ";
				@columns = ();
				push(@columns, $primary_key_values[0]);
			}
			push(@columns, $tmp[1]);
		}
		$keyword_string .= format_keywords($dbh,\@columns,$table)." ";
		debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
		check_data($dbh,$table,undef,$keyword_string,$stanza_name,"editable_update");
	}
	
	$statement =~ s/,$//g;
	$statement .= " where";

	my $pk_count = scalar(@primary_key_names);
	for(my $i = 0; $i < $pk_count; $i++)
	{
		$statement .= " ".$primary_key_names[$i]."='".$primary_key_values[$i]."' and";
	}
	$statement =~ s/and$//g;
	push(@update_statements, $statement);
	
	$json_str .= "OK";

}
elsif($type[1] eq "preempt")
{
#type=preempt&TLLS_CFGPreemptClass-incoming_class-12=bb&TLLS_CFGPreemptClass-method-12=sh&type=all_value&name=default&12-2-bb-all-sh=1&14-2-cc-all-su=1&13-2-www-enough-vc=1
#type=preempt&TLLS_CFGPreemptClass-incoming_class-14=cc&TLLS_CFGPreemptClass-method-14=su&TLLS_CFGPreemptClass-all_or_enough-14=all
	my $table;
	my %key_map = ();
	my $check = 0;
	for(my $i=1;$i<$kw_count;$i++)
	{
		if($keywords[$i] eq "type=all_value")
		{
			$check = $i;
			last;
		}
		my @tmp = split('=',$keywords[$i]);
		my $value = $tmp[1];
		@tmp = split('-',$tmp[0]);
		$table = $tmp[0];
		$key_map{$tmp[2]}{$tmp[1]}=$value;
	}

	# format the <keyword>,<value> pair for checking
	if($check > 0 && $kw_count > $check + 1)
	{
		my @tmp = split('=',$keywords[$check + 1]);
		my $stanza_name = $tmp[1];
		my $keyword_string = "";
		for(my $i = $check + 2; $i < $kw_count; $i++)
		{
			@tmp = split('=',$keywords[$i]);
			my @columns = split('-',$tmp[0]);
			$keyword_string .= format_keywords($dbh,\@columns, "TLLS_CFGPreemptClass");
		}

#		debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
		check_data($dbh,$table,undef,$keyword_string,$stanza_name,"preempt_update");
	}
	
	# format SQL update statement
	foreach my $id (keys %key_map)
	{
		my $statement = "update ".$table." set ";
		foreach my $key (keys %{$key_map{$id}})
		{
			$statement .= $key."='".$key_map{$id}{$key}."',";
		}
		
		$statement =~ s/,$//g;
		$statement .= " where preemptID=".$id;
		push(@update_statements, $statement);
	}

	$json_str .= "OK";
}
elsif($type[1] eq "start")
{
#type=start&TLLS_CFGPreemptStartClass-incoming_class-5-sa=aa&TLLS_CFGPreemptStartClass-running_class-5-sa=rr&type=all_value&name=default&5-sa-2-aa-rr-32=1&6-uuy-2-ewwe-tt-1=1&6-ye-2-ewwe-ye-333=1
#type=start&TLLS_CFGPreemptStartClass-running_class-6-uuy=tt&TLLS_CFGPreemptStartClass-number_of_tasks-6-uuy=1	
	my $table;
	my %key_map1 = ();
	my %key_map2 = ();
	my $check = 0;
	for(my $i=1;$i<$kw_count;$i++)
	{
		if($keywords[$i] eq "type=all_value")
		{
			$check = $i;
			last;
		}
		my @tmp = split('=',$keywords[$i]);
		my $value = $tmp[1];
		@tmp = split('-',$tmp[0]);
		$table = $tmp[0];
		if($tmp[1] eq "incoming_class")
		{
			$key_map1{$tmp[2]."-".$tmp[3]}{$tmp[1]}=$value;
		}
		else
		{
			$key_map2{$tmp[2]."-".$tmp[3]}{$tmp[1]}=$value;
		}
	}

	# format the <keyword>,<value> pair for checking
	if($check > 0 && $kw_count > $check + 1)
	{
		my @tmp = split('=',$keywords[$check + 1]);
		my $stanza_name = $tmp[1];
		my $keyword_string = "";
		my %start_class_map = ();
		for(my $i = $check + 2; $i < $kw_count; $i++)
		{
			@tmp = split('=',$keywords[$i]);
			my @columns = split('-',$tmp[0]);
			# 5-sa-2-aa-rr-32=1
			if(exists($start_class_map{$columns[3]}))
			{
				$start_class_map{$columns[3]} .= "(".$columns[4]."<".$columns[5].") && ";
			}
			else
			{
				$start_class_map{$columns[3]} = "(".$columns[4]."<".$columns[5].") && ";
			}
		}
		foreach my $incoming (keys %start_class_map)
		{
			$keyword_string .= "START_CLASS[".$incoming."] = ".$start_class_map{$incoming};
			$keyword_string =~ s/ && $/ /g;
		}
		
#		debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
		check_data($dbh,$table,undef,$keyword_string,$stanza_name,"start_update");
	}

	# format SQL update statement
	foreach my $id (keys %key_map1)
	{		
		my $statement = "update ".$table." set ";
		foreach my $key (keys %{$key_map1{$id}})
		{
			$statement .= $key."='".$key_map1{$id}{$key}."',";
		}
		$statement =~ s/,$//g;
		my @tmp = split('-',$id);
		$statement .= " where startID=".$tmp[0];
		push(@update_statements, $statement);
	}
	foreach my $id (keys %key_map2)
	{		
		my $statement = "update TLLS_CFGPreemptStartClassRunningTasks set ";
		foreach my $key (keys %{$key_map2{$id}})
		{
			$statement .= $key."='".$key_map2{$id}{$key}."',";
		}
		$statement =~ s/,$//g;
		my @tmp = split('-',$id);
		$statement .= " where startID=".$tmp[0]." and running_class='".$tmp[1]."'";
		push(@update_statements, $statement);
	}

	$json_str .= "OK";
}
else
{
	error_and_exit("Error: unknown type - ".$type[1]);
}

foreach my $sql (@update_statements)
{
#	debug_log($sql);
	if(!defined($dbh->do($sql)))
	{
		error_and_exit($dbh->errstr);
	}

}

$dbh->disconnect();


print $json_str;

==> LL_utils-5.1.0.11-x86_64-Linux-RHEL6.tar.gz: var/www/ll/cgi-bin/on_update_panel.pl <==
#!/usr/bin/perl

# static char sccsid[] = "@(#)20 1.12 src/ll/config_editor/cgi-bin/on_update_panel.pl, ll.config, ll_rrnep2, rrnep21206a 1/26/11 22:30:51\n";

use strict;
use lib "/var/www/ll/cgi-bin";
use LLUtil;
use DBI;

my $query = $ENV{QUERY_STRING};
$query = unescape($query);

debug_log($query);

# type=normal&pk_name=nodeID&pk_id=27&name=abc&TLLR_CFGMachine-adapter_list=33&TLLR_CFGMachine-alias=33&TLLR_CFGMachine-cpu_speed_scale=33
# type=editable&pk_name=nodeID-class_name&pk_id=27=a&table=TLLR_CFGMachineClass&class_count=11&type=all_value&name=abc&class_name=a&class_count=11&class_name=b&class_count=22	
my @keywords = split('&',$query);
my $kw_count = scalar(@keywords);
my @type = split('=',$keywords[0]);

# create the db connection
my $dbh = get_dbh(0);
if(!defined($dbh))
{
	error_and_exit(DBI->errstr);
}

my @update_statements = ();
my $json_str = LLUtil::OUTPUT_HEADER;

if($type[1] eq "normal")
{
# type=normal&pk_name=nodeID&pk_id=27&name=abc&TLLR_CFGMachine-adapter_list=33&TLLR_CFGMachine-alias=33&TLLR_CFGMachine-cpu_speed_scale=33    
	my @tmp = split('=',$keywords[1]);	
	my $pk_name = $tmp[1];
	@tmp = split('=',$keywords[2]);
	my $pk_id = $tmp[1];
	@tmp = split('=',$keywords[3]);
	my $stanza_name = $tmp[1];
	my $table;
	my %key_map = ();
	for(my $i=4;$i<$kw_count;$i++)
	{
		@tmp = split('=',$keywords[$i]);
		my $value = $tmp[1];
		@tmp = split('-',$tmp[0]);
		$table = $tmp[0];
		my $key = $tmp[1];
		
		check_data($dbh,$table,$key,$value,$stanza_name,"normal_update");
		$key_map{$tmp[1]}=$value;
	}

	# find out if the record exists
	my $sql_str="select ".$pk_name." from ".$table." where ".$pk_name."=".$pk_id;
	my $sth=$dbh->prepare($sql_str);
	if(!defined($sth->execute()))
	{
		error_and_exit($dbh->errstr);
	}
	my($id);
	$sth->bind_columns(undef,\$id);
	my $existed = 0;
	if($sth->fetch())
	{
		$existed = 1;
	}
	
	# format the update SQL statement
	my $statement;
	if($existed == 1)
	{
		$statement = "update ".$table." set ";
		foreach my $key (keys %key_map)
		{
			$statement .= $key."='".$key_map{$key}."',";
		}
		$statement =~ s/,$//g;
		$statement .= " where ".$pk_name."=".$pk_id;
	}
	else
	{
		my $ins_keys = "";
		my $ins_vals = "";
		foreach my $key (keys %key_map)
		{
			$ins_keys .= " ".$key.",";
			$ins_vals .= " '".$key_map{$key}."',";
		}
		$ins_keys =~ s/,$//g;
		$ins_vals =~ s/,$//g;
		$statement = "insert into ".$table." (".$pk_name.",".$ins_keys.") values('".$pk_id."',".$ins_vals.")";
	}
	
	push(@update_statements, $statement);
	
	$json_str .= "OK";
	
}
elsif($type[1] eq "editable")
{
# type=editable&pk_name=nodeID-class_name&pk_id=27=a&table=TLLR_CFGMachineClass&class_count=11&type=all_value&name=abc&class_name=a&class_count=11&class_name=b&class_count=22	
	my @tmp = split('=',$keywords[1]);
	my @primary_key_names = split('-',$tmp[1]);	
	my @primary_key_values = split('=',$keywords[2]);
	shift(@primary_key_values);
	@tmp = split('=',$keywords[3]);
	my $table = $tmp[1];

	# format the update SQL statement
	my $statement = "update ".$table." set ";
	my $check = 0;
	for(my $i = 4; $i < $kw_count; $i++)
	{
		if($keywords[$i] eq "type=all_value")
		{
			$check = $i;
			last;
		}
		
		@tmp = split('=',$keywords[$i]);
		$statement .= $tmp[0]."='".$tmp[1]."',";		
	}

	# format the <keyword>,<value> pair for checking
	if($check > 0 && $kw_count > $check + 1)
	{
		@tmp = split('=',$keywords[$check + 1]);
		my $stanza_name = $tmp[1];

		@tmp = split('=',$keywords[$check + 2]);
		my $first_key_name = $tmp[0];
		my @columns = ();
		push(@columns, $primary_key_values[0]);
		push(@columns, $tmp[1]);
		
		my $keyword_string = "";
		if (($table eq "TLLS_CFGClassLimits") || ($table eq "TLLR_CFGFileSysMonThresholds"))
		{
			# for keywords from table TLLS_CFGClassLimits or TLLR_CFGFileSysMonThresholds,
			# only check one key-value pair each loop.
			my $kw_name = "";
			for(my $i = $check+2; $i < $kw_count; $i++)
			{
				@tmp = split('=',$keywords[$i]);
				if($tmp[0] eq $first_key_name)
				{
					$keyword_string = format_keywords($dbh,\@columns,$table)." ";
					if ($kw_name ne "")
					{
						debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
						check_data($dbh,$table,$kw_name,$keyword_string,$stanza_name,"editable_update");
						$kw_name = "";
					}
					@columns = ();
					push(@columns, $primary_key_values[0]);
					$kw_name = $tmp[1];
					debug_log("on_update_panel.pl, kw_name - ".$kw_name);
				}
				push(@columns, $tmp[1]);
			}
			$kw_name = $columns[1];
			$keyword_string = format_keywords($dbh,\@columns,$table)." ";
			debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
			check_data($dbh,$table,$kw_name,$keyword_string,$stanza_name,"editable_update");
		}
		else
		{
			for(my $i = $check+2; $i < $kw_count; $i++)
			{
				@tmp = split('=',$keywords[$i]);
				if($tmp[0] eq $first_key_name)
				{
					$keyword_string .= format_keywords($dbh,\@columns,$table)." ";
					@columns = ();
					push(@columns, $primary_key_values[0]);
				}
				push(@columns, $tmp[1]);
			}
			$keyword_string .= format_keywords($dbh,\@columns,$table)." ";
			debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
			check_data($dbh,$table,undef,$keyword_string,$stanza_name,"editable_update");
		}
	}
	
	$statement =~ s/,$//g;
	$statement .= " where";

	my $pk_count = scalar(@primary_key_names);
	for(my $i = 0; $i < $pk_count; $i++)
	{
		$statement .= " ".$primary_key_names[$i]."='".$primary_key_values[$i]."' and";
	}
	$statement =~ s/and$//g;
	push(@update_statements, $statement);
	
	$json_str .= "OK";

}
elsif(lc($type[1]) eq "preempt")
{
#type=preempt&TLLS_CFGPreemptClass-incoming_class-12=bb&TLLS_CFGPreemptClass-method-12=sh&type=all_value&name=default&12-2-bb-all-sh=1&14-2-cc-all-su=1&13-2-www-enough-vc=1
#type=preempt&TLLS_CFGPreemptClass-incoming_class-14=cc&TLLS_CFGPreemptClass-method-14=su&TLLS_CFGPreemptClass-all_or_enough-14=all
	my $table;
	my %key_map = ();
	my $check = 0;
	for(my $i=1;$i<$kw_count;$i++)
	{
		if($keywords[$i] eq "type=all_value")
		{
			$check = $i;
			last;
		}
		my @tmp = split('=',$keywords[$i]);
		my $value = $tmp[1];
		@tmp = split('-',$tmp[0]);
		$table = $tmp[0];
		$key_map{$tmp[2]}{$tmp[1]}=$value;
	}

	# format the <keyword>,<value> pair for checking
	if($check > 0 && $kw_count > $check + 1)
	{
		my @tmp = split('=',$keywords[$check + 1]);
		my $stanza_name = $tmp[1];
		my $keyword_string = "";
		for(my $i = $check + 2; $i < $kw_count; $i++)
		{
			@tmp = split('=',$keywords[$i]);
			my @columns = split('-',$tmp[0]);
			$keyword_string .= format_keywords($dbh,\@columns, "TLLS_CFGPreemptClass");
		}

#		debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
		check_data($dbh,$table,undef,$keyword_string,$stanza_name,"preempt_update");
	}
	
	# format SQL update statement
	foreach my $id (keys %key_map)
	{
		my $statement = "update ".$table." set ";
		foreach my $key (keys %{$key_map{$id}})
		{
			$statement .= $key."='".$key_map{$id}{$key}."',";
		}
		
		$statement =~ s/,$//g;
		$statement .= " where preemptID=".$id;
		push(@update_statements, $statement);
	}

	$json_str .= "OK";
}
elsif(lc($type[1]) eq "start")
{
#type=start&TLLS_CFGPreemptStartClass-incoming_class-5-sa=aa&TLLS_CFGPreemptStartClass-running_class-5-sa=rr&type=all_value&name=default&5-sa-2-aa-rr-32=1&6-uuy-2-ewwe-tt-1=1&6-ye-2-ewwe-ye-333=1
#type=start&TLLS_CFGPreemptStartClass-running_class-6-uuy=tt&TLLS_CFGPreemptStartClass-number_of_tasks-6-uuy=1	
	my $table;
	my %key_map1 = ();
	my %key_map2 = ();
	my $check = 0;
	for(my $i=1;$i<$kw_count;$i++)
	{
		if($keywords[$i] eq "type=all_value")
		{
			$check = $i;
			last;
		}
		my @tmp = split('=',$keywords[$i]);
		my $value = $tmp[1];
		@tmp = split('-',$tmp[0]);
		$table = $tmp[0];
		if(lc($tmp[1]) eq "incoming_class")
		{
			$key_map1{$tmp[2]."-".$tmp[3]}{$tmp[1]}=$value;
		}
		else
		{
			$key_map2{$tmp[2]."-".$tmp[3]}{$tmp[1]}=$value;
		}
	}

	# format the <keyword>,<value> pair for checking
	if($check > 0 && $kw_count > $check + 1)
	{
		my @tmp = split('=',$keywords[$check + 1]);
		my $stanza_name = $tmp[1];
		my $keyword_string = "";
		my %start_class_map = ();
		for(my $i = $check + 2; $i < $kw_count; $i++)
		{
			@tmp = split('=',$keywords[$i]);
			my @columns = split('-',$tmp[0]);
			# 5-sa-2-aa-rr-32=1
			if(exists($start_class_map{$columns[3]}))
			{
				$start_class_map{$columns[3]} .= "(".$columns[4]."<".$columns[5].") && ";
			}
			else
			{
				$start_class_map{$columns[3]} = "(".$columns[4]."<".$columns[5].") && ";
			}
		}
		foreach my $incoming (keys %start_class_map)
		{
			$keyword_string .= "START_CLASS[".$incoming."] = ".$start_class_map{$incoming};
			$keyword_string =~ s/ && $/ /g;
		}
		
#		debug_log("on_update_panel.pl, keyword_string - ".$keyword_string);
		check_data($dbh,$table,undef,$keyword_string,$stanza_name,"start_update");
	}

	# format SQL update statement
	foreach my $id (keys %key_map1)
	{		
		my $statement = "update ".$table." set ";
		foreach my $key (keys %{$key_map1{$id}})
		{
			$statement .= $key."='".$key_map1{$id}{$key}."',";
		}
		$statement =~ s/,$//g;
		my @tmp = split('-',$id);
		$statement .= " where startID=".$tmp[0];
		push(@update_statements, $statement);
	}
	foreach my $id (keys %key_map2)
	{		
		my $statement = "update TLLS_CFGPreemptStartClassRunningTasks set ";
		foreach my $key (keys %{$key_map2{$id}})
		{
			$statement .= $key."='".$key_map2{$id}{$key}."',";
		}
		$statement =~ s/,$//g;
		my @tmp = split('-',$id);
		$statement .= " where startID=".$tmp[0]." and running_class='".$tmp[1]."'";
		push(@update_statements, $statement);
	}

	$json_str .= "OK";
}
else
{
	error_and_exit("Error: unknown type - ".$type[1]);
}

foreach my $sql (@update_statements)
{
#	debug_log($sql);
	if(!defined($dbh->do($sql)))
	{
		error_and_exit($dbh->errstr);
	}

}

$dbh->disconnect();


print $json_str;
